
/*
    Program tsubplan

    Test specul routine with an actual planet. 
*/

#include "SpiceUsr.h"
#include "SpiceZmc.h" 
#include "SpiceEll.h"

#define   ROOM     40
#define   SUNID    10 

int main()
{


   SpiceBoolean   found;
 
   SpiceChar      frname    [ROOM];
  
   SpiceDouble    alt;
   SpiceDouble    arad;
   SpiceDouble    brad;
   SpiceDouble    crad;
   SpiceDouble    dpr;
   SpiceDouble    et; 
   SpiceDouble    flat; 
   SpiceDouble    lat; 
   SpiceDouble    lon; 
   SpiceDouble    lt; 
   SpiceDouble    mat       [3][3]; 
   SpiceDouble    obspos    [3]; 
   SpiceDouble    spang;   
   SpiceDouble    specpoint [3];   
   SpiceDouble    state     [6];   
   SpiceDouble    sunang; 
   SpiceDouble    sun2000   [3];
   SpiceDouble    sunpos    [3];
   SpiceDouble    values    [3];
   SpiceDouble    xpt       [3];
   SpiceDouble    zero      [] = {0.0, 0.0, 0.0};

   SpiceInt       dim;
   SpiceInt       frcode; 
   SpiceInt       ier; 
   SpiceInt       niter;
   SpiceInt       scid; 
   SpiceInt       targid; 

/*
   Read in metakernel.
*/
 
   furnsh_c ( "tsubplan.ker" );

/*
   Input target body's NAIF ID
*/

   printf( "Enter target body NAIF ID:\n" );
   scanf( "%d", &targid );
   printf( "targid = %d\n", targid );

/*
   Input spacecraft ID
*/

   printf( "Input spacecraft's NAIF ID code:\n" );
   scanf( "%d", &scid ); 
   printf( "scid = %d\n", scid );


/*
   Input et in seconds from 2000.
*/
 
   printf( "Input observation time, ET seconds from 2000\n" );
   scanf( "%lf", &et );
 
   printf( "observation time = %f \n", et );
 
/*
   Get radii of target body
*/
  
   bodvar_c ( targid, "RADII", &dim, values );
 
   printf("after bodvar_c: dim = %d \n", dim ); 

   arad = values[0];
   brad = values[1];
   crad = values[2];

   printf("arad = %f, brad = %f, crad = %f \n", arad, brad, crad );

   flat = (arad - crad) / arad;
 
   printf( "flat = %f\n", flat);
   
/*
   Get name of target's body-fixed frame.   
*/

   cidfrm_c ( targid, ROOM, &frcode, frname, &found );

   printf ( "after cidfrm_c\n");
   printf ( "targid = %d\n", targid );

   if ( found == SPICETRUE ) { 

      printf( "frcode = %d\n", frcode );
      printf( "frname = %s\n", frname );  

   } else { 
    
      printf( "frame code not found\n" );
      exit(1);

   } 
   

/*
   Get body-fixed vector from target to Sun
*/

   spkez_c ( SUNID, et, frname, "LT+S", targid, state, &lt ); 
 
   vpack_c ( state[0], state[1], state[2], sunpos );
 
   printf("sunpos = %e %e %e\n", sunpos[0], sunpos[1], sunpos[2] );
 
   pxform_c ( frname, "J2000", et, mat ); 

/*
   Output to Sun vector in J2000 frame
*/
 
   mxv_c ( mat, sunpos, sun2000 );

   printf("sun2000 = %e %e %e\n", sun2000[0], sun2000[1], sun2000[2] );
   
/*
   determine planetodetic lat and lon of subsolar point. 
*/

      surfpt_c ( zero, sunpos, arad, brad, crad, xpt, &found );

      recgeo_c ( xpt, arad, flat, &lon, &lat, &alt );

      lat = lat * dpr_c();
      lon = lon * dpr_c();

      if ( lon < 0.0 ) lon = 360.0 + lon;

      printf( "subsolar lat = %f, lon = %f deg.\n", lat,  lon );

/*
   Get body-fixed vector from target to spacecraft (observer).
*/

   spkez_c ( scid, et, frname, "LT+S", targid, state, &lt ); 

   vpack_c ( state[0], state[1], state[2], obspos );

   printf( "obspos = %e %e %e\n", obspos[0], obspos[1], obspos[2] );

/*
   determine planetodetic lat and lon of subspacecraft point. 
*/

      surfpt_c ( zero, obspos, arad, brad, crad, xpt, &found );

      recgeo_c (xpt, arad, flat, &lon, &lat, &alt );

      lat = lat * dpr_c();
      lon = lon * dpr_c();

      if ( lon < 0.0 ) lon = 360.0 + lon;

      printf( "subspacecraft lat = %f, lon = %f deg.\n", lat,  lon );

  
/*
   Attempt to locate the specular point.
*/ 

   specul ( arad,
            brad,
            crad, 
            sunpos,
            obspos,
            specpoint,
           &spang, 
           &found,
           &ier     ); 


   if ( found == SPICETRUE ) 
   {
      printf("specular point found at = %f %f %f\n", 
              specpoint[0], specpoint[1], specpoint[2]);
 
/*
   determine planetodetic lat and lon of specular point
*/

      recgeo_c ( specpoint, arad, flat, &lon, &lat, &alt );

      lat = lat * dpr_c();
      lon = lon * dpr_c();

      if ( lon < 0.0 ) lon = 360.0 + lon;

      printf( "specular lat = %f, lon = %f deg.\n", lat,  lon );
      
                                                          
      printf("spang = %f\n", spang*dpr_c() );
      printf("number of iterations  = %d\n", ier);

   } else {

      printf("specular point not found\n");
      printf("ier = %d\n", ier);

   }
   exit(0);
}   
